<?php
/**
 * @file
 * Contains CommercePostmasterAdmin.
 *
 * This class provides the functions required to administrator the
 * commerce_postmaster module
 */

class CommercePostmasterAdmin {

  /**
   * Function shippingOptionsForm.
   *
   * Shipping options for internal shipping rates. This is used when creating
   * tracking ID\'s through the postmaster API.
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form state array.
   *
   * @return form
   *   The rendered form array.
   */
  public static function shippingOptionsForm(array $form, array &$form_state) {

    $form['#tree'] = TRUE;

    // This fieldset just displays more details about the other form elements.
    $form['legend'] = array(
      '#type' => 'fieldset',
      '#title' => t('Shipping Options Legend'),
      '#description' => t('This relates to the service levels from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#services'))),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $header = array(
      t('Postmaster Name'),
      t('Fedex'), t('UPS'),
      t('USPS'),
      t('LSO'),
      t('Canada Post'),
    );
    $rows = array();
    $rows[] = array(
      t('GROUND'),
      t('Ground/Home Delivery'),
      t('3 Day Select'),
      t('Priority or 1st Class'),
      t('Ground Early'),
      t('Regular Parcel'),
    );
    $rows[] = array(
      t('3DAY'),
      t('Express Saver'),
      t('Ground'),
      t('Parcel Select or 1st Class'),
      t('Ground Basic'),
      t('Expedited Parcel'),
    );
    $rows[] = array(
      t('2DAY'),
      t('2Day'),
      t('2nd Day Air'),
      t('Priority'),
      t('Priority Noon'),
      t('Priority'),
    );
    $rows[] = array(
      t('2DAY_EARLY'),
      t('2Day AM'),
      t('2nd Day Air AM'),
      t('Express'),
      t('Priority 2nd Day'),
      t('Priority'),
    );
    $rows[] = array(
      t('1DAY'),
      t('Standard Overnight'),
      t('Next Day Air Saver'),
      t('Express'),
      t('Priority Basic'),
      t('Xpresspost'),
    );
    $rows[] = array(
      t('1DAY_EARLY'),
      t('Priority Overnight'),
      t('Next Day Air'),
      t('Express'),
      t('Priority Early'),
      t('Xpresspost'),
    );
    $rows[] = array(
      t('1DAY_MORNING'),
      t('First Overnight'),
      t('Next Day Air Early A.M.'),
      t('Express'),
      t('Priority Basic'),
      t('Xpresspost'),
    );
    $markup = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'class' => array('cpm_local_shipping'),
        'id' => array('cpm_local_shipping'),
      ),
    ));
    $form['legend']['sl_table'] = array(
      '#markup' => $markup,
    );

    $rows = array();
    $rows[] = array(
      t('INTL_SURFACE'),
      t('N/A'),
      t('N/A'),
      t('1st Class International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_PRIORITY'),
      t('International Economy'),
      t('Worldwide Saver'),
      t('Priority) International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_EXPRESS'),
      t('International Priority'),
      t('Worldwide Express'),
      t('Express) International'),
      t('N/A'),
      t('N/A'),
    );
    $markup = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'class' => array('cpm_local_shipping'),
        'id' => array('cpm_local_shipping'),
      ),
    ));

    $form['legend']['sl_intl_table'] = array(
      '#markup' => $markup,
    );

    $rows = array();
    $rows[] = array(
      t('INTL_SURFACE'),
      t('N/A'),
      t('N/A'),
      t('1st Class International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_PRIORITY'),
      t('International Economy'),
      t('Worldwide Saver'),
      t('Priority) International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_EXPRESS'),
      t('International Priority'),
      t('Worldwide Express'),
      t('Express) International'),
      t('N/A'),
      t('N/A'),
    );
    $markup = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'class' => array('cpm_packaging'),
        'id' => array('cpm_packaging'),
      ),
    ));
    $form['legend']['packaging_table'] = array(
      '#markup' => $markup,
    );

    $rows = array();
    $rows[] = array(
      t('INTL_SURFACE'),
      t('N/A'),
      t('N/A'),
      t('1st Class International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_PRIORITY'),
      t('International Economy'),
      t('Worldwide Saver'),
      t('Priority) International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_EXPRESS'),
      t('International Priority'),
      t('Worldwide Express'),
      t('Express) International'),
      t('N/A'),
      t('N/A'),
    );
    $markup = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'class' => array('cpm_label_formats'),
        'id' => array('cpm_label_formats'),
      ),
    ));
    $form['legend']['label_formats_table'] = array(
      '#markup' => $markup,
    );

    $rows = array();
    $rows[] = array(
      t('INTL_SURFACE'),
      t('N/A'),
      t('N/A'),
      t('1st Class International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_PRIORITY'),
      t('International Economy'),
      t('Worldwide Saver'),
      t('Priority) International'),
      t('N/A'),
      t('N/A'),
    );
    $rows[] = array(
      t('INTL_EXPRESS'),
      t('International Priority'),
      t('Worldwide Express'),
      t('Express) International'),
      t('N/A'),
      t('N/A'),
    );
    $markup = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'class' => array('cpm_label_sizes'),
        'id' => array('cpm_label_sizes'),
      ),
    ));
    $form['legend']['label_sizes_table'] = array(
      '#markup' => $markup,
    );

    // This is where you set the site wide defaults.
    $form['defaults'] = array(
      '#type' => 'fieldset',
      '#title' => t('Default Shipping Option Settings'),
      '#description' => t('This relates to the service levels from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#services'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['defaults']['service_levels'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default service level shipping options'),
      '#description' => t('This relates to the service levels from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#services'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    // Set the local service level options.
    $form['defaults']['service_levels']['sl_local'] = array(
      '#type' => 'select',
      '#title' => t('Local Service Levels'),
      '#options' => self::getServiceLevelArray(),
      '#default_value' => variable_get('commerce_postmaster_sl_local', 'GROUND'),
      '#description' => t('Set the default Local service level for all parcels.'),
    );
    // Set the international service level options.
    $form['defaults']['service_levels']['sl_intl'] = array(
      '#type' => 'select',
      '#title' => t('International Service Levels'),
      '#options' => self::getIntServiceLevelArray(),
      '#default_value' => variable_get('commerce_postmaster_sl_intl', 'INTL_SURFACE'),
      '#description' => t('Set the default International service level for all parcels.'),
    );

    $form['defaults']['packaging'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default Packaging shipping options'),
      '#description' => t('This relates to the Packaging from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#packages'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    // Set the packaging options.
    $form['defaults']['packaging']['packaging'] = array(
      '#type' => 'select',
      '#title' => t('Packaging'),
      '#options' => self::getPackageOptionArray(),
      '#default_value' => variable_get('commerce_postmaster_packaging', 'CARRIER_BOX_SMALL'),
      '#description' => t('Set the default International service level for all parcels.'),
    );

    $form['defaults']['label_formats'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default Label formats shipping options'),
      '#description' => t('This relates to the Label formats from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#label-formats'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['defaults']['label_formats']['label_formats'] = array(
      '#type' => 'select',
      '#title' => t('Label Formats'),
      '#options' => self::getLabelFormatArray(),
      '#default_value' => variable_get('commerce_postmaster_label_formats', 'PNG'),
      '#description' => t('Set the default Label Formats for all parcels.'),
    );

    // Set the label_sizes options.
    $form['defaults']['label_formats']['label_sizes'] = array(
      '#type' => 'select',
      '#title' => t('Label Sizes'),
      '#options' => self::getLabelSizeArray(),
      '#default_value' => variable_get('commerce_postmaster_label_sizes', 'SMALL'),
      '#description' => t('Set the default Label Sizes for all parcels.'),
    );

    $form['defaults']['dimensions'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default dimension shipping options'),
      '#description' => t('This relates to the weight, height, length and width that will be used when creating shipping items'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    // Dimensions.
    $form['defaults']['dimensions']['commerce_postmaster_dim_weight'] = array(
      '#type' => 'textfield',
      '#title' => t('Weight'),
      '#default_value' => variable_get('commerce_postmaster_dim_weight', 1),
      '#description' => t('Set the default weight for all parcels.'),
    );
    $form['defaults']['dimensions']['commerce_postmaster_dim_height'] = array(
      '#type' => 'textfield',
      '#title' => t('Height'),
      '#default_value' => variable_get('commerce_postmaster_dim_height', 1),
      '#description' => t('Set the default height for all parcels.'),
    );
    $form['defaults']['dimensions']['commerce_postmaster_dim_length'] = array(
      '#type' => 'textfield',
      '#title' => t('Length'),
      '#default_value' => variable_get('commerce_postmaster_dim_length', 1),
      '#description' => t('Set the default length for all parcels.'),
    );
    $form['defaults']['dimensions']['commerce_postmaster_dim_width'] = array(
      '#type' => 'textfield',
      '#title' => t('Width'),
      '#default_value' => variable_get('commerce_postmaster_dim_width', 1),
      '#description' => t('Set the default width for all parcels.'),
    );

    // Enable overrides per order.
    $form['orders'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enable per order shipping options'),
      '#description' => t('Enable overrides for the above values for each order. Order based overrides will also override product specific shipping options.'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    $form['orders']['service_levels'] = array(
      '#type' => 'checkbox',
      '#default_value' => variable_get('commerce_postmaster_order_sl', 0),
      '#title' => t('Order Service Levels'),
    );

    $form['orders']['service_levels_intl'] = array(
      '#type' => 'checkbox',
      '#default_value' => variable_get('commerce_postmaster_order_sli', 0),
      '#title' => t('Order International Service Levels'),
    );

    $form['orders']['packaging'] = array(
      '#type' => 'checkbox',
      '#default_value' => variable_get('commerce_postmaster_order_pak', 0),
      '#title' => t('Order Packages'),
    );

    $form['orders']['label_formats'] = array(
      '#type' => 'checkbox',
      '#default_value' => variable_get('commerce_postmaster_order_lf', 0),
      '#title' => t('Order Label Formats'),
    );

    $form['orders']['label_sizes'] = array(
      '#type' => 'checkbox',
      '#default_value' => variable_get('commerce_postmaster_order_ls', 0),
      '#title' => t('Order Label Sizes'),
    );
    // Add in order weights and dimensions.
    $form['orders']['dimensions'] = array(
      '#type' => 'checkbox',
      '#title' => t('Dimensions'),
      '#default_value' => variable_get('commerce_postmaster_order_dim', 0),
    );

    /*
     * Loop through all of the products, create a fieldset then add fields to it
     * for each item.
     */
    $form['prod_types'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enable per product shipping options'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    foreach (commerce_product_types() as $type => $product_type) {
      $form['prod_types'][$type] = array(
        '#type' => 'fieldset',
        '#title' => t('@name (@machine_name)', array('@name' => $product_type['name'], '@machine_name' => $type)),
        '#description' => t('Configure shipping options settings for all @product_name (@type) products. Note that disabling service levels management removes the service levels field from the product type, deleting any existing service levels data for that product type.', array('@product_name' => $product_type['name'], '@type' => $type)),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      );

      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_service_level', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_service_levels'] = array(
        '#type' => 'checkbox',
        '#title' => t('Service levels'),
        '#default_value' => $enabled[$type],
      );

      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_intl_service_level', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_service_levels_intl'] = array(
        '#type' => 'checkbox',
        '#title' => t('International service levels'),
        '#default_value' => $enabled[$type],
      );

      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_packaging', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_packaging'] = array(
        '#type' => 'checkbox',
        '#title' => t('Packaging Options'),
        '#default_value' => $enabled[$type],
      );

      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_label_formats', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_label_formats'] = array(
        '#type' => 'checkbox',
        '#title' => t('Label Formats'),
        '#default_value' => $enabled[$type],
      );

      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_label_sizes', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_label_sizes'] = array(
        '#type' => 'checkbox',
        '#title' => t('Label Sizes'),
        '#default_value' => $enabled[$type],
      );

      // Add in product weights and dimensions.
      $instance[$type] = field_info_instance('commerce_product', 'commerce_pm_weight', $type);
      $enabled[$type] = (!empty($instance[$type]));
      $form['prod_types'][$type]['prod_type_dimensions'] = array(
        '#type' => 'checkbox',
        '#title' => t('Dimensions'),
        '#default_value' => $enabled[$type],
      );
    }

    // Add a checkbox that requires them to say "I do", but don't show it
    // (#access == FALSE) unless they're deleting.
    $visible = FALSE;
    if (!empty($form_state['commerce_ship_option']['delete_instances'])) {
      $type_plural = format_plural(count($form_state['commerce_ship_option']['delete_instances']), 'type', 'types');
      $affirmation = t('I understand that all field data will be permanently removed from the product @type_plural %product_types.',
        array(
          '@type_plural' => $type_plural,
          '%product_types' => implode(', ', $form_state['commerce_ship_option']['delete_instances']),
        )
      );
      $visible = TRUE;
    }

    $form['error'] = array(
      '#type' => 'fieldset',
      '#title' => t('Confirm deleting your field'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#access' => $visible,
      '#weight' => -40,
    );

    $form['error']['confirmation'] = array(
      '#type' => 'checkbox',
      '#title' => !empty($affirmation) ? $affirmation : '',
      '#default_value' => FALSE,
      '#access' => FALSE,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Submit'),
    );

    // If they're deleting, show the confirmation checkbox.
    if (!empty($form_state['commerce_ship_option']['delete_instances'])) {
      $form['error']['confirmation']['#access'] = TRUE;
      drupal_set_message(t('You must click the confirmation checkbox to confirm that you want to delete shipping options data'), 'warning');
    }

    return $form;
  }

  /**
   * Function shippingOptionsFormValidate.
   *
   * Form validator. If they are deleting and have not checked the confirmation
   * checkbox, make them do so.
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function shippingOptionsFormValidate(array $form, array &$form_state) {
    if (!empty($form_state['commerce_ship_option']['delete_instances']) && empty($form_state['values']['error']['confirmation'])) {
      form_set_error('error', t('Please check the "I understand" checkbox to indicate you understand that all shipping options data in these fields will be deleted: %fields.', array('%fields' => implode(', ', $form_state['commerce_ship_option']['delete_instances']))));
    }
    // Ensure the dimension are numeric
    $dimensions = '';
    if (!is_numeric($form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_weight'])) {
      $dimensions = 'defaults][dimensions][commerce_postmaster_dim_weight';
      form_set_error($dimensions);
    }
    if (!is_numeric($form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_height'])) {
      $dimensions = 'defaults][dimensions][commerce_postmaster_dim_height';
      form_set_error($dimensions);
    }
    if (!is_numeric($form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_length'])) {
      $dimensions = 'defaults][dimensions][commerce_postmaster_dim_length';
      form_set_error($dimensions);
    }
    if (!is_numeric($form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_width'])) {
      $dimensions = 'defaults][dimensions][commerce_postmaster_dim_width';
      form_set_error($dimensions);
    }
    if ($dimensions != '') {
      drupal_set_message(t('The dimensions must be numeric'), 'error');
    }
  }

  /**
   * Function shippingOptionsFormSubmit.
   *
   * Add or remove the Stock field from product types.
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function shippingOptionsFormSubmit(array $form, array &$form_state) {
    // Set the basic default variables.
    variable_set('commerce_postmaster_sl_local', $form_state['values']['defaults']['service_levels']['sl_local']);
    variable_set('commerce_postmaster_sl_intl', $form_state['values']['defaults']['service_levels']['sl_intl']);
    variable_set('commerce_postmaster_packaging', $form_state['values']['defaults']['packaging']['packaging']);
    variable_set('commerce_postmaster_label_formats', $form_state['values']['defaults']['label_formats']['label_formats']);
    variable_set('commerce_postmaster_label_sizes', $form_state['values']['defaults']['label_formats']['label_sizes']);
    variable_set('commerce_postmaster_dim_weight', $form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_weight']);
    variable_set('commerce_postmaster_dim_height', $form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_height']);
    variable_set('commerce_postmaster_dim_length', $form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_length']);
    variable_set('commerce_postmaster_dim_width', $form_state['values']['defaults']['dimensions']['commerce_postmaster_dim_width']);
    // Create the delete array.
    $form_state['commerce_ship_option']['delete_instances'] = array();

    // Set the local service level options.
    $default = t('GROUND');
    // This will be used as the description for the field.
    $description = t('This relates to the service levels from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#services')));
    $field_name = 'commerce_pm_service_level';
    $label = 'Shipping Service Level';
    $prod_fields = 'prod_type_service_levels';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['service_levels'];
    variable_set('commerce_postmaster_order_sl', $order_fields);
    self::createFields($field_name, $label, self::getServiceLevelArray(), $default, $description, $order_fields, $prod_fields, $form_state);

    // Set the international service level options.
    $default = t('INTL_SURFACE');
    // This will be used as the description for the field.
    $description = t('This relates to the international service levels from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#services')));
    $field_name = 'commerce_pm_intl_service_level';
    $label = 'International Shipping Service Level';
    $prod_fields = 'prod_type_service_levels_intl';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['service_levels_intl'];
    variable_set('commerce_postmaster_order_sli', $order_fields);
    self::createFields($field_name, $label, self::getIntServiceLevelArray(), $default, $description, $order_fields, $prod_fields, $form_state);

    // Set the packaging options.
    $default = t('CARRIER_BOX_SMALL');
    // This will be used as the description for the field.
    $description = t('This relates to the prod_type_packaging from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#packages')));
    $field_name = 'commerce_pm_packaging';
    $label = 'Shipping Packages';
    $prod_fields = 'prod_type_packaging';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['packaging'];
    variable_set('commerce_postmaster_order_pak', $order_fields);
    // Set up the order fields.
    self::createFields($field_name, $label, self::getPackageOptionArray(), $default, $description, $order_fields, $prod_fields, $form_state);

    // Set the label_formats options.
    $default = t('PNG');
    // This will be used as the description for the field.
    $description = t('This relates to the label formats from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#label-formats')));
    $field_name = 'commerce_pm_label_formats';
    $label = 'Shipping Label Formats';
    $prod_fields = 'prod_type_label_formats';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['label_formats'];
    variable_set('commerce_postmaster_order_lf', $form_state['values']['orders']['label_formats']);
    self::createFields($field_name, $label, self::getLabelFormatArray(), $default, $description, $order_fields, $prod_fields, $form_state);

    // Set the label_sizes options.
    $default = t('SMALL');
    // This will be used as the description for the field.
    $description = t('This relates to the label sizes from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#label-sizes')));
    $field_name = 'commerce_pm_label_sizes';
    $label = 'Shipping Label Sizes';
    $prod_fields = 'prod_type_label_sizes';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['label_sizes'];
    variable_set('commerce_postmaster_order_ls', $form_state['values']['orders']['label_sizes']);
    self::createFields($field_name, $label, self::getLabelSizeArray(), $default, $description, $order_fields, $prod_fields, $form_state);

    // Set the weight options.
    $options = array();
    $default = '0';
    // This will be used as the description for the field.
    $description = t('The weight of the weight of the product, this is used to calculate shipping weights');
    $field_name = 'commerce_pm_weight';
    $label = 'Shipping Weight';
    $prod_fields = 'prod_type_dimensions';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['dimensions'];
    self::createFields($field_name, $label, $options, $default, $description, $order_fields, $prod_fields, $form_state, 'text');
    $description = t('The height of the height of the product, this is used to calculate shipping dimensions');
    $field_name = 'commerce_pm_height';
    $label = 'Shipping Height';
    $prod_fields = 'prod_type_dimensions';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['dimensions'];
    self::createFields($field_name, $label, $options, $default, $description, $order_fields, $prod_fields, $form_state, 'text');
    $description = t('The length of the length of the product, this is used to calculate shipping dimensions');
    $field_name = 'commerce_pm_length';
    $label = 'Shipping length';
    $prod_fields = 'prod_type_dimensions';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['dimensions'];
    self::createFields($field_name, $label, $options, $default, $description, $order_fields, $prod_fields, $form_state, 'text');
    $description = t('The width of the width of the product, this is used to calculate shipping dimensions');
    $field_name = 'commerce_pm_width';
    $label = 'Shipping width';
    $prod_fields = 'prod_type_dimensions';
    // Set up the order fields.
    $order_fields = $form_state['values']['orders']['dimensions'];
    self::createFields($field_name, $label, $options, $default, $description, $order_fields, $prod_fields, $form_state, 'text');
    variable_set('commerce_postmaster_order_dim', $form_state['values']['orders']['dimensions']);
  }

  /**
   * Function checkTrackingForm.
   *
   * Check Tracking form, this is a basic form and forms
   * an example of how to integrate this into your own
   * form.
   */
  public static function checkTrackingForm() {
    $form = array();
    $form['status'] = array(
      '#markup' => "<div id='result'>Enter a Shipment Tracking ID to see it's current status<div>",
      '#weight' => -20,
    );
    $form['postmaster_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Shipment Tracking ID'),
      '#size' => 12,
      '#maxlength' => 12,
      '#required' => TRUE,
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#title' => t('Check Tracking'),
      '#value' => t('Check tracking'),
    );

    return $form;
  }

  /**
   * Function checkTrackingFormValidate.
   *
   * Validate the check tracking form
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function checkTrackingFormValidate(array &$form, array &$form_state) {
    if ($form_state['values']['postmaster_id'] == '') {
      form_set_error('postmaster_id', t("The Shipment Order ID can't be blank."));
    }
  }

  /**
   * Function checkTrackingFormSubmit.
   *
   * Call the check tracking function and return the results
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function checkTrackingFormSubmit(array $form, array $form_state) {
    $response = CommercePostmaster::checkTracking($form_state['values']['postmaster_id']);
    $data = $response;

    if (isset($data['status'])) {
      $message = '<table style="width:40%"><tr><td width="20%">Status:</td><td>' .
        $data['status'] . '</td></tr>';
      $message = '<tr><td colspan="2"><b>History</b></td></tr>';
      foreach ($data['history'] as $item) {
        $message = '<tr><td>Status:</td><td>' . $item['status'] . '</td></tr>';
        $message = '<tr><td>Time:</td><td>' . format_date($item['timestamp']) . '</td></tr>';
        $message = '<tr><td>Description:</td><td>' . $item['description'] . '</td></tr>';
      }
      $message .= '<tr><td>Last Update:</td><td>' . format_date($data['last_update']) . '</td></tr></table>';
      $message = filter_xss($message, array('td', 'tr', 'table', 'b'));
      drupal_set_message($message);
    }
    else {
      drupal_set_message(t('Tracking ID not found'), 'error');
    }

    return $form;
  }

  /**
   * Function validateAddressForm.
   *
   * Validate an address, this is a basic form and forms
   * an example of how to integrate this into your own
   * form. This could be greatly improved by integrating it
   * with the address field module rather than using these
   * basic fields
   */
  public static function validateAddressForm() {
    $form = array();
    $form['status'] = array(
      '#markup' => '<div id="result">' . t('Enter an address you would like to validate.') . '<div>',
      '#weight' => "-20",
    );
    $form['company'] = array(
      '#type' => 'textfield',
      '#title' => t('Company'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['contact'] = array(
      '#type' => 'textfield',
      '#title' => t('Contact'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );

    $form['line1'] = array(
      '#type' => 'textfield',
      '#title' => t('Address'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['line2'] = array(
      '#type' => 'textfield',
      '#title' => t('Address Cont..'),
      '#size' => 60,
      '#maxlength' => 128,
    );
    $form['line3'] = array(
      '#type' => 'textfield',
      '#title' => t('Address Cont..'),
      '#size' => 60,
      '#maxlength' => 128,
    );
    $form['city'] = array(
      '#type' => 'textfield',
      '#title' => t('City'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['state'] = array(
      '#type' => 'textfield',
      '#title' => t('State'),
      '#size' => 10,
      '#maxlength' => 10,
      '#required' => TRUE,
    );
    $form['zip_code'] = array(
      '#type' => 'textfield',
      '#title' => t('Zip'),
      '#size' => 10,
      '#maxlength' => 10,
      '#required' => TRUE,
    );
    $form['country'] = array(
      '#type' => 'select',
      '#title' => t('Country Code'),
      '#options' => CommercePostmaster::countryGetList(),
      '#required' => TRUE,
      '#default_value' => 'US',
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#title' => t('Validate Address'),
      '#value' => t('Validate Address'),
    );

    return $form;
  }

  /**
   * Function validateAddressFormSubmit.
   *
   * Validate an address.
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function validateAddressFormSubmit(array &$form, array &$form_state) {
    $form_values = $form_state['values'];
    // Known good array structure - for some dumb reason the api fails
    // validation with extra fields use array_intersect_key to remove extra
    // Drupal value fields.
    $good_address = array(
      "company" => NULL,
      "contact" => NULL,
      "line1" => NULL,
      "line2" => NULL,
      "line3" => NULL,
      "city" => NULL,
      "state" => NULL,
      "zip_code" => NULL,
      "country" => NULL,
    );
    $form_values = array_intersect_key($form_values, $good_address);
    $results = CommercePostmaster::validateAddress($form_values);

    $message = '<table style="width:40%"><tr><td width="20%">Status:</td><td>' .
      $results['status'] . '</td></tr>
      <tr><td>Notes</td><td>' . $results['addresses'][0]['notes'] .
      '</td></tr></table>';
    $message = filter_xss($message, array('td', 'tr', 'table', 'b'));
    drupal_set_message($message);

    return $form;
  }

  /**
   * Function shippingForm.
   *
   * Validate an address, this is a basic form and forms
   * an example of how to integrate this into your own
   * form.
   */
  public static function shippingForm() {
    $form = array();
    // Package info.
    $form['package'] = array(
      '#type' => 'fieldset',
      '#title' => t('Package'),
      '#weight' => 0,
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    );
    $form['package']['weight'] = array(
      '#type' => 'textfield',
      '#title' => t('Weight'),
      '#size' => 6,
      '#maxlength' => 6,
      '#default_value' => 10,
      '#required' => TRUE,
    );
    $form['package']['length'] = array(
      '#type' => 'textfield',
      '#title' => t('Length'),
      '#size' => 6,
      '#maxlength' => 6,
      '#default_value' => 10,
      '#required' => TRUE,
    );

    $form['package']['width'] = array(
      '#type' => 'textfield',
      '#title' => t('Width'),
      '#size' => 6,
      '#maxlength' => 6,
      '#default_value' => 10,
      '#required' => TRUE,
    );
    $form['package']['height'] = array(
      '#type' => 'textfield',
      '#title' => t('Height'),
      '#size' => 6,
      '#maxlength' => 6,
      '#default_value' => 10,
      '#required' => TRUE,
    );

    // End package info.
    $form['company'] = array(
      '#type' => 'textfield',
      '#title' => t('Company'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['contact'] = array(
      '#type' => 'textfield',
      '#title' => t('Contact'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );

    $form['line1'] = array(
      '#type' => 'textfield',
      '#title' => t('Address'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['line2'] = array(
      '#type' => 'textfield',
      '#title' => t('Address Cont..'),
      '#size' => 60,
      '#maxlength' => 128,
    );
    $form['line3'] = array(
      '#type' => 'textfield',
      '#title' => t('Address Cont..'),
      '#size' => 60,
      '#maxlength' => 128,
    );
    $form['city'] = array(
      '#type' => 'textfield',
      '#title' => t('City'),
      '#size' => 60,
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['state'] = array(
      '#type' => 'textfield',
      '#title' => t('State'),
      '#size' => 10,
      '#maxlength' => 10,
      '#required' => TRUE,
    );
    $form['zip_code'] = array(
      '#type' => 'textfield',
      '#title' => t('Zip'),
      '#size' => 10,
      '#maxlength' => 10,
      '#required' => TRUE,
    );
    $form['country'] = array(
      '#type' => 'select',
      '#title' => t('Country Code'),
      '#options' => CommercePostmaster::countryGetList(),
      '#required' => TRUE,
      '#default_value' => 'US',
    );
    $form['phone_no'] = array(
      '#type' => 'textfield',
      '#title' => t('Phone No'),
      '#size' => 60,
      '#maxlength' => 128,
    );
    // Set the carrier options.
    $options = array(
      'fedex' => 'FedEx',
      'ups' => t('UPS'),
      'usps' => t('USPS'),
    );
    $form['carrier'] = array(
      '#type' => 'select',
      '#title' => t('Carrier'),
      '#options' => $options,
    );

    // Set the local service level options.
    $form['sl_local'] = array(
      '#type' => 'select',
      '#title' => t('Local Service Levels'),
      '#options' => self::getServiceLevelArray(),
      '#default_value' => variable_get('commerce_postmaster_sl_local', 'GROUND'),
      '#description' => t('Set the default Local service level for all parcels.'),
    );
    // Set the international service level options.
    $form['sl_intl'] = array(
      '#type' => 'select',
      '#title' => t('International Service Levels'),
      '#options' => self::getIntServiceLevelArray(),
      '#default_value' => variable_get('commerce_postmaster_sl_intl', 'INTL_SURFACE'),
      '#description' => t('Set the default International service level for all parcels.'),
    );

    $form['packaging'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default Packaging shipping options'),
      '#description' => t('This relates to the Packaging from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#packages'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    // Set the packaging options.
    $form['packaging'] = array(
      '#type' => 'select',
      '#title' => t('Packaging'),
      '#options' => self::getPackageOptionArray(),
      '#default_value' => variable_get('commerce_postmaster_packaging', 'CARRIER_BOX_SMALL'),
      '#description' => t('Set the default International service level for all parcels.'),
    );

    $form['label_formats'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enter default Label formats shipping options'),
      '#description' => t('This relates to the Label formats from postmaster which can be found !link.', array('!link' => l(t('here'), 'https://www.postmaster.io/docs#label-formats'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    // Set the label_formats options.
    $form['label_formats'] = array(
      '#type' => 'select',
      '#title' => t('Label Formats'),
      '#options' => self::getLabelFormatArray(),
      '#default_value' => variable_get('commerce_postmaster_label_formats', 'PNG'),
      '#description' => t('Set the default Label Formats for all parcels.'),
    );

    // Set the label_formats options.
    $form['label_sizes'] = array(
      '#type' => 'select',
      '#title' => t('Label Sizes'),
      '#options' => self::getLabelSizeArray(),
      '#default_value' => variable_get('commerce_postmaster_label_sizes', 'SMALL'),
      '#description' => t('Set the default Label Sizes for all parcels.'),
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#title' => t('Validate Address'),
      '#value' => t('Validate Address'),
    );

    return $form;
  }

  /**
   * Function shippingFormSubmit.
   *
   * Validate an address
   *
   * @param array $form
   *   Drupal form array.
   * @param array $form_state
   *   Drupal form_state array.
   */
  public static function shippingFormSubmit(array &$form, array &$form_state) {
    $form_values = $form_state['values'];
    $address = array(
      "company" => NULL,
      "contact" => NULL,
      "line1" => NULL,
      "line2" => NULL,
      "line3" => NULL,
      "city" => NULL,
      "state" => NULL,
      "zip_code" => NULL,
      "country" => NULL,
      "phone_no" => NULL,
    );
    $addr_values = array_intersect_key($form_values, $address);
    $package = array(
      'weight' => NULL,
      'length' => NULL,
      'width' => NULL,
      'height' => NULL,
    );
    $package = array_intersect_key($form_values['package'], $package);
    $package['customs'] = array();
    $package['customs']['type'] = t('Other');
    $package['customs']['comments'] = t('Drupal CM Test Package');
    $customs = array(
      'description' => t('Drupal CM Test Package'),
      'country_of_origin' => variable_get('commerce_postmaster_country_origin', t('US')),
      'quantity' => "1",
      'value' => "10",
      'weight' => $form_values['package']['weight'],
    );
    $package['customs']['contents'][0] = $customs;
    $shipment = array();
    $shipment['to'] = $addr_values;
    $shipment['package'] = $package;
    $shipment['carrier'] = $form_values['carrier'];
    $shipment['service'] = $form_values['country'] == 'US' ? $form_values['sl_local'] : $form_values['sl_intl'];
    $formats = $form_values['label_formats'];
    $types = array(
      'EPL' => t('THERMAL'),
      'ZPL2' => t('THERMAL'),
      'PNG' => t('NORMAL'),
      'GIF' => t('NORMAL'),
      'PDF' => t('NORMAL'),
    );
    $shipment['packaging'] = $form_values['packaging'];
    $shipment['label'] = array(
      'type' => $types[$formats],
      'format' => $formats,
      'size' => $form_values['label_sizes'],
    );
    $results = CommercePostmaster::createShipment($shipment);

    $message = '<table style="width:40%"><tr><td width="20%">Status:</td><td>' .
      $results['status'] . '</td></tr>
      <tr><td>Tracking ID:</td><td>' . $results['tracking'][0] .
      '</td></tr></table>';
    $message = filter_xss($message, array('td', 'tr', 'table', 'b'));
    drupal_set_message($message);

    return $form;
  }

  /**
   * Function createFields.
   *
   * Take in field details and create them in both a commerce_order and
   * commerce_product.
   *
   * @param string $field_name
   *   The machine name for the field.
   * @param string $label
   *   The field label.
   * @param string $options
   *   An array of options for select fields.
   * @param string $default
   *   The field default value.
   * @param string $description
   *   The field description.
   * @param string $order_fields
   *   An array from the form state indicating whether to create or remove this
   *   field.
   * @param string $prod_field
   *   An array from the form state indicating whether to create or remove this
   *   field.
   * @param string $form_state
   *   The form_state array.
   * @param string $field_type
   *   The type of field, the machine name based on the form API.
   */
  protected static function createFields($field_name, $label, $options, $default, $description, $order_fields, $prod_field, &$form_state, $field_type = 'list_text') {
    $weights = array(
      'commerce_pm_service_level' => 1,
      'commerce_pm_intl_service_level' => 2,
      'commerce_pm_packaging' => 3,
      'commerce_pm_label_formats' => 4,
      'commerce_pm_label_sizes' => 5,
      'commerce_pm_weight' => 6,
      'commerce_pm_height' => 7,
      'commerce_pm_length' => 8,
      'commerce_pm_width' => 9,
      'field_cm_tracking_id' => 59,
    );
    $resubmit = self::checkField('commerce_order', 'commerce_order', $order_fields, $field_name, $label, $default, $description, $weights[$field_name], $options, $form_state['values']['error']['confirmation'], $field_type);
    // If the function returned false, show the please confirm checkbox.
    if (!$resubmit) {
      $form_state['commerce_ship_option']['delete_instances'][] = 'commerce_order';
      $form_state['rebuild'] = TRUE;
    }

    // Loop through the product types for the service level options.
    foreach (commerce_product_types() as $type => $product_type) {
      // Create or delete the field as required, if it's marked for deletion and
      // the user has not checked the approve box, return false.
      $resubmit = self::checkField('commerce_product', $type, $form_state['values']['prod_types'][$type][$prod_field], $field_name, $label, $default, $description, $weights[$field_name], $options, $form_state['values']['error']['confirmation'], $field_type);
      // If the function returned false, show the please confirm checkbox.
      if (!$resubmit) {
        $form_state['commerce_ship_option']['delete_instances'][] = $label . ' (' . $type . ')';
        $form_state['rebuild'] = TRUE;
      }
    }
  }

  /**
   * Check if a field exists and if it doesn't, create it.
   *
   * @param string $entity
   *   The type of entity to attach the field to.
   * @param string $type
   *   The type of field to create.
   * @param string $enable
   *   Ff the user wants to enable or disable this field.
   * @param string $field
   *   The name of the field to be created.
   * @param string $label
   *   The field label.
   * @param string $default
   *   The default value for the field.
   * @param string $description
   *   The field description.
   * @param string $weight
   *   The field weight.
   * @param string $options
   *   The field options if it has any.
   * @param string $confirmed
   *   If the user has confirmed deleting the field.
   * @param  string $field_type
   *   The type of field, defaults to list_text
   *
   * @return bool
   *   True if the delete option has been approved.
   */
  protected static function checkField($entity, $type, $enable, $field, $label, $default, $description, $weight, $options, $confirmed, $field_type = 'list_text') {
    // Does the field exist.
    $instance = field_info_instance($entity, $field, $type);

    // Is it currently enabled.
    $currently_enabled = self::productTypeEnabled($entity, $type, $field);

    // If they want us to enable it and it doesn't currently exist, do the work.
    if ($enable && !$currently_enabled) {
      self::createInstance($field, $field_type, TRUE, $entity, $type, $label, $default, $options, $description, $weight);
      // Initialize fields of every existing product of $type bundle to 0.
      switch ($entity) {
        case 'commerce_product':
          self::productFieldInit($entity, $type, $field, $default);
          break;

        case 'commerce_order':
          self::orderFieldInit($entity, $type, $field, $default);
          break;

      }
      drupal_set_message(t('%label has been enabled on the %type product type', array('%type' => $type, '%label' => $label)));
    }
    // Conversely, if they *don't* want it and it's currently enabled,
    // warn them about the consequences or do it.
    elseif (!$enable && $currently_enabled) {
      // If they haven't clicked the "confirm" checkbox, rebuild and get them
      // to do it.
      if (empty($confirmed)) {
        return FALSE;
      }
      // Otherwise they already have clicked it and we can delete.
      else {
        // Remove the instance.
        field_delete_instance($instance);

        drupal_set_message(t('%label management has been disabled on the %type product type', array('%type' => $type, '%label' => $label)));
      }
    }
    return TRUE;
  }

  /**
   * Function createInstance.
   *
   * Creates a field and an instance of a commerce_postmaster field on the
   * specified bundle.
   *
   * @param string $field_name
   *   The name of the field; if it already exists, a new instance of the
   *   existing field will be created. For fields governed by the Commerce
   *   modules, this should begin with commerce_.
   * @param string $field_type
   *   The type of field to create.
   * @param bool $required
   *   If this field is a required field.
   * @param string $entity_type
   *   The type of entity the field instance will be attached to..
   * @param string $bundle
   *   The bundle name of the entity the field instance will be attached to.
   * @param string $label
   *   The label of the field instance.
   * @param string $default
   *   The default value for this field.
   * @param array $options
   *   An array of field options for select lists..
   * @param string $description
   *   The field description.
   * @param int $weight
   *   The weight of the field.
   * @param bool $no_display
   *   Hide the field display.
   */
  public static function createInstance($field_name, $field_type,
          $required, $entity_type, $bundle, $label, $default, $options = array(),
          $description = NULL, $weight = 0, $no_display = TRUE) {

    // Look for the field and instance.
    $field = field_info_field($field_name);
    $instance = field_info_instance($entity_type, $field_name, $bundle);

    // If there is no field then create one based on the passed in values.
    if (empty($field)) {
      $field = array(
        'field_name' => $field_name,
        'type' => $field_type,
        'cardinality' => 1,
        'translatable' => FALSE,
        'locked' => FALSE,
        'default_value' => array(array('value' => $default)),
      );
      // If it shouldn't be displayed hide it.
      if ($no_display) {
        $field['display'] = array(
          'default' => array(
            'label' => 'above',
            'type' => 'hidden',
          ),
        );
      }
      // If there are passed in options, create them for the select field.
      if (count($options)) {
        $field['settings'] = array(
          'allowed_values' => $options,
          'allowed_values_function' => '',
        );
        $field['widget'] = array(
          'type' => 'select',
        );
      }

      // If the field type is boolean, set the values.
      if ($field_type == 'list_boolean') {
        $field['settings'] = array(
          'allowed_values' => array(0, 1),
          'allowed_values_function' => '',
        );
      }
      field_create_field($field);
    }

    // If there is no field instance then create one.
    if (empty($instance)) {
      $instance = array(
        'field_name' => $field_name,
        'entity_type' => $entity_type,
        'bundle' => $bundle,
        'label' => $label,
        'required' => $required,
        'widget' => array(
          'weight' => $weight,
        ),
        'settings' => array(),
        'description' => $description,
        'default_value' => array(array('value' => $default)),
      );

      // If it shouldn't be displayed hide it.
      if ($no_display) {
        $instance['display'] = array(
          'default' => array(
            'label' => 'above',
            'type' => 'hidden',
          ),
        );
      }

      // If it's a boolean st it's allowed values.
      if ($field_type == 'list_boolean') {
        $instance['widget'] = array(
          'type' => 'options_onoff',
          'settings' => array(
            'display_label' => TRUE,
          ),
        );
      }

      // Create the field instance.
      field_create_instance($instance);
    }

  }

  /**
   * Given the field data, determine whether field exists for that entity.
   *
   * @param string $entity
   *   The entity type name.
   * @param string $type
   *   The field type.
   * @param string $field
   *   The name of the field.
   *
   * @return bool
   *   True if the field exists.
   */
  protected static function productTypeEnabled($entity, $type, $field) {
    $instance = field_info_instance($entity, $field, $type);
    return (!empty($instance));
  }

  /**
   * Funciton productFieldInit.
   *
   * This functions parses all the product of the specified bundle, and
   * initialize the required field to the default value.
   *
   * @param string $entity
   *   The entity type name.
   * @param string $bundle
   *   The bundle with the entity.
   * @param string $field_name
   *   The name of the field.
   * @param string $default
   *   The field default value.
   */
  protected static function productFieldInit($entity, $bundle, $field_name, $default) {
    // Load every product of $bundle type.
    $products = commerce_product_load_multiple(array(), array('type' => $bundle));
    // Parses each product to initialize its stock value.
    foreach ($products as $product) {
      $wrapper = entity_metadata_wrapper($entity, $product);
      $wrapper->$field_name = $default;
      $wrapper->save();
    }
  }

  /**
   * Function orderFieldInit.
   *
   * This functions parses all the order of the specified bundle, and initialize
   * the require field.
   *
   * @param string $entity
   *   The entity type name.
   * @param string $field_name
   *   The name of the field.
   * @param string $default
   *   The field default value.
   */
  protected static function orderFieldInit($entity, $field_name, $default) {
    $query = db_select('commerce_order', 'cod')
      ->fields('cod', array('order_id'))
      ->execute();

    foreach ($query->fetchCol() as $order_id) {
      // Load every product of $bundle type.
      $order = commerce_order_load($order_id);
      $wrapper = entity_metadata_wrapper($entity, $order);
      $wrapper->$field_name = $default;
      $wrapper->save();
    }
  }

  /**
   * Returns a static array of Service Levels, mainly for forms.
   *
   * @return array
   *    The array used mainly in forms.
   */
  public static function getServiceLevelArray() {
    return array(
      'GROUND' => t('GROUND'),
      '3DAY' => t('3DAY'),
      '2DAY' => t('2DAY'),
      '2DAY_EARLY' => t('2DAY_EARLY'),
      '1DAY' => t('1DAY'),
      '1DAY_EARLY' => t('1DAY_EARLY'),
      '1DAY_MORNING' => t('1DAY_MORNING'),
    );
  }

  /**
   * Returns a static array of International Service Levels, mainly for forms.
   *
   * @return array
   *    The array used mainly in forms.
   */
  public static function getIntServiceLevelArray() {
    return array(
      'INTL_SURFACE' => t('INTL_SURFACE'),
      'INTL_PRIORITY' => t('INTL_PRIORITY'),
      'INTL_EXPRESS' => t('INTL_EXPRESS'),
    );
  }

  /**
   * Returns a static array of packaging options, mainly for forms.
   *
   * @return array
   *    The array used mainly in forms.
   */
  public static function getPackageOptionArray() {
    return array(
      'TUBE' => t('TUBE'),
      'LETTER' => t('LETTER'),
      'PAK' => t('PAK'),
      'CARRIER_BOX_SMALL' => t('CARRIER_BOX_SMALL'),
      'CARRIER_BOX_MEDIUM' => t('CARRIER_BOX_MEDIUM'),
      'CARRIER_BOX_LARGE' => t('CARRIER_BOX_LARGE'),
      'CUSTOM' => t('CUSTOM'),
    );
  }

  /**
   * Returns a static array of label formats, mainly for forms.
   *
   * @return array
   *    The array used mainly in forms.
   */
  public static function getLabelFormatArray() {
    return array(
      'EPL' => t('EPL'),
      'ZPL2' => t('ZPL2'),
      'PNG' => t('PNG'),
      'GIF' => t('GIF'),
      'PDF' => t('PDF'),
    );
  }

  /**
   * Returns a static array of label sizes, mainly for forms.
   *
   * @return array
   *    The array used mainly in forms.
   */
  public static function getLabelSizeArray() {
    return array(
      'SMALL' => t('SMALL'),
      'MEDIUM' => t('MEDIUM'),
      'LARGE' => t('LARGE'),
    );
  }
}
